@@ -1,8 +1,25 @@
+# -*- python-mode -*-
+
+## Copyright (C) 2012-2013  Daniel Pavel
+##
+## This program is free software; you can redistribute it and/or modify
+## it under the terms of the GNU General Public License as published by
+## the Free Software Foundation; either version 2 of the License, or
+## (at your option) any later version.
+##
+## This program is distributed in the hope that it will be useful,
+## but WITHOUT ANY WARRANTY; without even the implied warranty of
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License along
+## with this program; if not, write to the Free Software Foundation, Inc.,
+## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
 import errno as _errno
+import logging
 import threading as _threading
 
-from logging import INFO as _INFO
-from logging import getLogger
 from typing import Optional
 
 import hidapi as _hid
@@ -15,8 +32,7 @@ from . import hidpp20 as _hidpp20
 from .common import strhex as _strhex
 from .settings_templates import check_feature_settings as _check_feature_settings
 
-_log = getLogger(__name__)
-del getLogger
+logger = logging.getLogger(__name__)
 
 _R = _hidpp10.REGISTERS
 _IR = _hidpp10.INFO_SUBREGISTERS
@@ -60,10 +76,9 @@ class Device:
         if receiver:
             assert number > 0 and number <= 15  # some receivers have devices past their max # of devices
         self.number = number  # will be None at this point for directly connected devices
-        self.online = None
+        self.online = self.descriptor = None
 
         self.wpid = None  # the Wireless PID is unique per device model
-        self.descriptor = None
         self._kind = None  # mouse, keyboard, etc (see _hidpp10.DEVICE_KIND)
         self._codename = None  # Unifying peripherals report a codename.
         self._name = None  # the full name of the model
@@ -74,17 +89,13 @@ class Device:
         self._tid_map = None  # map from transports to product identifiers
         self._persister = None  # persister holds settings
 
-        self._firmware = None
-        self._keys = None
-        self._remap_keys = None
-        self._gestures = None
+        self._firmware = self._keys = self._remap_keys = self._gestures = None
+        self._polling_rate = self._power_switch = self._led_effects = None
+
         self._gestures_lock = _threading.Lock()
-        self._registers = None
-        self._settings = None
+        self._profiles = self._backlight = self._registers = self._settings = None
         self._feature_settings_checked = False
         self._settings_lock = _threading.Lock()
-        self._polling_rate = None
-        self._power_switch = None
 
         # See `add_notification_handler`
         self._notification_handlers = {}
@@ -116,7 +127,7 @@ class Device:
             elif receiver.receiver_kind == '27Mhz':  # 27 Mhz receiver doesn't have pairing registers
                 self.wpid = _hid.find_paired_node_wpid(receiver.path, number)
                 if not self.wpid:
-                    _log.error('Unable to get wpid from udev for device %d of %s', number, receiver)
+                    logger.error('Unable to get wpid from udev for device %d of %s', number, receiver)
                     raise _base.NoSuchDevice(number=number, receiver=receiver, error='Not present 27Mhz device')
                 kind = receiver.get_kind_from_index(number)
                 self._kind = _hidpp10.DEVICE_KIND[kind]
@@ -195,7 +206,10 @@ class Device:
     def name(self):
         if not self._name:
             if not self.online:  # be very defensive
-                self.ping()
+                try:
+                    self.ping()
+                except _base.NoSuchDevice:
+                    pass
             if self.online and self.protocol >= 2.0:
                 self._name = _hidpp20.get_name(self)
         return self._name or self._codename or ('Unknown device %s' % (self.wpid or self.product_id))
@@ -204,8 +218,8 @@ class Device:
         ids = _hidpp20.get_ids(self)
         if ids:
             self._unitId, self._modelId, self._tid_map = ids
-            if _log.isEnabledFor(_INFO) and self._serial and self._serial != self._unitId:
-                _log.info('%s: unitId %s does not match serial %s', self, self._unitId, self._serial)
+            if logger.isEnabledFor(logging.INFO) and self._serial and self._serial != self._unitId:
+                logger.info('%s: unitId %s does not match serial %s', self, self._unitId, self._serial)
 
     @property
     def unitId(self):
@@ -233,7 +247,7 @@ class Device:
             if not self._kind:
                 self._kind = kind
             if not self._polling_rate:
-                self._polling_rate = polling_rate
+                self._polling_rate = str(polling_rate) + 'ms'
 
     def update_extended_pairing_information(self):
         if self.receiver:
@@ -289,6 +303,12 @@ class Device:
             self._polling_rate = rate if rate else self._polling_rate
         return self._polling_rate
 
+    @property
+    def led_effects(self):
+        if not self._led_effects and self.online and self.protocol >= 2.0:
+            self._led_effects = _hidpp20.LEDEffectsInfo(self)
+        return self._led_effects
+
     @property
     def keys(self):
         if not self._keys:
@@ -312,6 +332,20 @@ class Device:
                         self._gestures = _hidpp20.get_gestures(self) or ()
         return self._gestures
 
+    @property
+    def backlight(self):
+        if self._backlight is None:
+            if self.online and self.protocol >= 2.0:
+                self._backlight = _hidpp20.get_backlight(self)
+        return self._backlight
+
+    @property
+    def profiles(self):
+        if self._profiles is None:
+            if self.online and self.protocol >= 2.0:
+                self._profiles = _hidpp20.get_profiles(self)
+        return self._profiles
+
     @property
     def registers(self):
         if not self._registers:
@@ -390,12 +424,12 @@ class Device:
             set_flag_bits = 0
         ok = _hidpp10.set_notification_flags(self, set_flag_bits)
         if not ok:
-            _log.warn('%s: failed to %s device notifications', self, 'enable' if enable else 'disable')
+            logger.warning('%s: failed to %s device notifications', self, 'enable' if enable else 'disable')
 
         flag_bits = _hidpp10.get_notification_flags(self)
         flag_names = None if flag_bits is None else tuple(_hidpp10.NOTIFICATION_FLAG.flag_names(flag_bits))
-        if _log.isEnabledFor(_INFO):
-            _log.info('%s: device notifications %s %s', self, 'enabled' if enable else 'disabled', flag_names)
+        if logger.isEnabledFor(logging.INFO):
+            logger.info('%s: device notifications %s %s', self, 'enabled' if enable else 'disabled', flag_names)
         return flag_bits if ok else None
 
     def add_notification_handler(self, id: str, fn):
@@ -415,8 +449,8 @@ class Device:
     def remove_notification_handler(self, id: str):
         """Unregisters the notification handler under name `id`."""
 
-        if id not in self._notification_handlers and _log.isEnabledFor(_INFO):
-            _log.info(f'Tried to remove nonexistent notification handler {id} from device {self}.')
+        if id not in self._notification_handlers and logger.isEnabledFor(logging.INFO):
+            logger.info(f'Tried to remove nonexistent notification handler {id} from device {self}.')
         else:
             del self._notification_handlers[id]
 
@@ -478,9 +512,11 @@ class Device:
     __nonzero__ = __bool__
 
     def __str__(self):
-        return '<Device(%d,%s,%s,%s)>' % (
-            self.number, self.wpid or self.product_id, self.name or self.codename or '?', self.serial
-        )
+        try:
+            name = self.name or self.codename or '?'
+        except _base.NoSuchDevice:
+            name = 'name not available'
+        return '<Device(%d,%s,%s,%s)>' % (self.number, self.wpid or self.product_id, name, self.serial)
 
     __repr__ = __str__
 
@@ -506,11 +542,11 @@ class Device:
                     bus_id=device_info.bus_id
                 )
         except OSError as e:
-            _log.exception('open %s', device_info)
+            logger.exception('open %s', device_info)
             if e.errno == _errno.EACCES:
                 raise
         except Exception:
-            _log.exception('open %s', device_info)
+            logger.exception('open %s', device_info)
 
     def close(self):
         handle, self.handle = self.handle, None
