@@ -17,24 +17,22 @@
 # Handles incoming events from the receiver/devices, updating the related
 # status object as appropriate.
 
+import logging
 import threading as _threading
 
-from logging import DEBUG as _DEBUG
-from logging import INFO as _INFO
-from logging import getLogger
 from struct import unpack as _unpack
 
 from . import diversion as _diversion
 from . import hidpp10 as _hidpp10
 from . import hidpp20 as _hidpp20
+from . import settings_templates as _st
 from .base import DJ_MESSAGE_ID as _DJ_MESSAGE_ID
 from .common import strhex as _strhex
 from .i18n import _
 from .status import ALERT as _ALERT
 from .status import KEYS as _K
 
-_log = getLogger(__name__)
-del getLogger
+logger = logging.getLogger(__name__)
 
 _R = _hidpp10.REGISTERS
 _F = _hidpp20.FEATURE
@@ -72,8 +70,8 @@ def _process_receiver_notification(receiver, status, n):
     if n.sub_id == 0x4A:  # pairing lock notification
         status.lock_open = bool(n.address & 0x01)
         reason = (_('pairing lock is open') if status.lock_open else _('pairing lock is closed'))
-        if _log.isEnabledFor(_INFO):
-            _log.info('%s: %s', receiver, reason)
+        if logger.isEnabledFor(logging.INFO):
+            logger.info('%s: %s', receiver, reason)
         status[_K.ERROR] = None
         if status.lock_open:
             status.new_device = None
@@ -81,7 +79,7 @@ def _process_receiver_notification(receiver, status, n):
         if pair_error:
             status[_K.ERROR] = error_string = _hidpp10.PAIRING_ERRORS[pair_error]
             status.new_device = None
-            _log.warn('pairing error %d: %s', pair_error, error_string)
+            logger.warning('pairing error %d: %s', pair_error, error_string)
         status.changed(reason=reason)
         return True
 
@@ -89,8 +87,8 @@ def _process_receiver_notification(receiver, status, n):
         with notification_lock:
             status.discovering = n.address == 0x00
             reason = (_('discovery lock is open') if status.discovering else _('discovery lock is closed'))
-            if _log.isEnabledFor(_INFO):
-                _log.info('%s: %s', receiver, reason)
+            if logger.isEnabledFor(logging.INFO):
+                logger.info('%s: %s', receiver, reason)
             status[_K.ERROR] = None
             if status.discovering:
                 status.counter = status.device_address = status.device_authentication = status.device_name = None
@@ -98,7 +96,7 @@ def _process_receiver_notification(receiver, status, n):
             discover_error = ord(n.data[:1])
             if discover_error:
                 status[_K.ERROR] = discover_string = _hidpp10.BOLT_PAIRING_ERRORS[discover_error]
-                _log.warn('bolt discovering error %d: %s', discover_error, discover_string)
+                logger.warning('bolt discovering error %d: %s', discover_error, discover_string)
             status.changed(reason=reason)
             return True
 
@@ -123,8 +121,8 @@ def _process_receiver_notification(receiver, status, n):
             status.device_passkey = None
             status.lock_open = n.address == 0x00
             reason = (_('pairing lock is open') if status.lock_open else _('pairing lock is closed'))
-            if _log.isEnabledFor(_INFO):
-                _log.info('%s: %s', receiver, reason)
+            if logger.isEnabledFor(logging.INFO):
+                logger.info('%s: %s', receiver, reason)
             status[_K.ERROR] = None
             if not status.lock_open:
                 status.counter = status.device_address = status.device_authentication = status.device_name = None
@@ -136,7 +134,7 @@ def _process_receiver_notification(receiver, status, n):
             if pair_error:
                 status[_K.ERROR] = error_string = _hidpp10.BOLT_PAIRING_ERRORS[pair_error]
                 status.new_device = None
-                _log.warn('pairing error %d: %s', pair_error, error_string)
+                logger.warning('pairing error %d: %s', pair_error, error_string)
             status.changed(reason=reason)
             return True
 
@@ -148,7 +146,7 @@ def _process_receiver_notification(receiver, status, n):
     elif n.sub_id == _R.passkey_pressed_notification:  # Bolt pairing
         return True
 
-    _log.warn('%s: unhandled notification %s', receiver, n)
+    logger.warning('%s: unhandled notification %s', receiver, n)
 
 
 #
@@ -189,63 +187,63 @@ def _process_device_notification(device, status, n):
 
     # assuming 0x00 to 0x3F are feature (HID++ 2.0) notifications
     if not device.features:
-        _log.warn('%s: feature notification but features not set up: %02X %s', device, n.sub_id, n)
+        logger.warning('%s: feature notification but features not set up: %02X %s', device, n.sub_id, n)
         return False
     try:
         feature = device.features.get_feature(n.sub_id)
     except IndexError:
-        _log.warn('%s: notification from invalid feature index %02X: %s', device, n.sub_id, n)
+        logger.warning('%s: notification from invalid feature index %02X: %s', device, n.sub_id, n)
         return False
 
     return _process_feature_notification(device, status, n, feature)
 
 
 def _process_dj_notification(device, status, n):
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('%s (%s) DJ %s', device, device.protocol, n)
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('%s (%s) DJ %s', device, device.protocol, n)
 
     if n.sub_id == 0x40:
         # do all DJ paired notifications also show up as HID++ 1.0 notifications?
-        if _log.isEnabledFor(_INFO):
-            _log.info('%s: ignoring DJ unpaired: %s', device, n)
+        if logger.isEnabledFor(logging.INFO):
+            logger.info('%s: ignoring DJ unpaired: %s', device, n)
         return True
 
     if n.sub_id == 0x41:
         # do all DJ paired notifications also show up as HID++ 1.0 notifications?
-        if _log.isEnabledFor(_INFO):
-            _log.info('%s: ignoring DJ paired: %s', device, n)
+        if logger.isEnabledFor(logging.INFO):
+            logger.info('%s: ignoring DJ paired: %s', device, n)
         return True
 
     if n.sub_id == 0x42:
         connected = not n.address & 0x01
-        if _log.isEnabledFor(_INFO):
-            _log.info('%s: DJ connection: %s %s', device, connected, n)
+        if logger.isEnabledFor(logging.INFO):
+            logger.info('%s: DJ connection: %s %s', device, connected, n)
         status.changed(active=connected, alert=_ALERT.NONE, reason=_('connected') if connected else _('disconnected'))
         return True
 
-    _log.warn('%s: unrecognized DJ %s', device, n)
+    logger.warning('%s: unrecognized DJ %s', device, n)
 
 
 def _process_hidpp10_custom_notification(device, status, n):
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('%s (%s) custom notification %s', device, device.protocol, n)
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('%s (%s) custom notification %s', device, device.protocol, n)
 
     if n.sub_id in (_R.battery_status, _R.battery_charge):
         # message layout: 10 ix <register> <xx> <yy> <zz> <00>
         assert n.data[-1:] == b'\x00'
         data = chr(n.address).encode() + n.data
-        charge, status_text, next_charge = _hidpp10.parse_battery_status(n.sub_id, data)
-        status.set_battery_info(charge, next_charge, status_text, None)
+        charge, next_charge, status_text, voltage = _hidpp10.parse_battery_status(n.sub_id, data)
+        status.set_battery_info(charge, next_charge, status_text, voltage)
         return True
 
     if n.sub_id == _R.keyboard_illumination:
         # message layout: 10 ix 17("address")  <??> <?> <??> <light level 1=off..5=max>
         # TODO anything we can do with this?
-        if _log.isEnabledFor(_INFO):
-            _log.info('illumination event: %s', n)
+        if logger.isEnabledFor(logging.INFO):
+            logger.info('illumination event: %s', n)
         return True
 
-    _log.warn('%s: unrecognized %s', device, n)
+    logger.warning('%s: unrecognized %s', device, n)
 
 
 def _process_hidpp10_notification(device, status, n):
@@ -260,7 +258,7 @@ def _process_hidpp10_notification(device, status, n):
                 del device.receiver[device.number]
             status.changed(active=False, alert=_ALERT.ALL, reason=_('unpaired'))
         else:
-            _log.warn('%s: disconnection with unknown type %02X: %s', device, n.address, n)
+            logger.warning('%s: disconnection with unknown type %02X: %s', device, n.address, n)
         return True
 
     # device connection (and disconnection)
@@ -275,12 +273,12 @@ def _process_hidpp10_notification(device, status, n):
             link_established = not (flags & 0x40)
             link_encrypted = bool(flags & 0x20) or n.address == 0x10  # Bolt protocol always encrypted
         else:
-            _log.warn('%s: connection notification with unknown protocol %02X: %s', device.number, n.address, n)
+            logger.warning('%s: connection notification with unknown protocol %02X: %s', device.number, n.address, n)
             return True
         if wpid != device.wpid:
-            _log.warn('%s wpid mismatch, got %s', device, wpid)
-        if _log.isEnabledFor(_DEBUG):
-            _log.debug(
+            logger.warning('%s wpid mismatch, got %s', device, wpid)
+        if logger.isEnabledFor(logging.DEBUG):
+            logger.debug(
                 '%s: protocol %s connection notification: software=%s, encrypted=%s, link=%s, payload=%s', device, n.address,
                 bool(flags & 0x10), link_encrypted, link_established, bool(flags & 0x80)
             )
@@ -298,53 +296,55 @@ def _process_hidpp10_notification(device, status, n):
     # power notification
     if n.sub_id == 0x4B:
         if n.address == 0x01:
-            if _log.isEnabledFor(_DEBUG):
-                _log.debug('%s: device powered on', device)
+            if logger.isEnabledFor(logging.DEBUG):
+                logger.debug('%s: device powered on', device)
             reason = status.to_string() or _('powered on')
             status.changed(active=True, alert=_ALERT.NOTIFICATION, reason=reason)
         else:
-            _log.warn('%s: unknown %s', device, n)
+            logger.warning('%s: unknown %s', device, n)
         return True
 
-    _log.warn('%s: unrecognized %s', device, n)
+    logger.warning('%s: unrecognized %s', device, n)
 
 
 def _process_feature_notification(device, status, n, feature):
-    if _log.isEnabledFor(_DEBUG):
-        _log.debug('%s: notification for feature %s, report %s, data %s', device, feature, n.sub_id >> 4, _strhex(n.data))
+    if logger.isEnabledFor(logging.DEBUG):
+        logger.debug('%s: notification for feature %s, report %s, data %s', device, feature, n.address >> 4, _strhex(n.data))
 
     if feature == _F.BATTERY_STATUS:
         if n.address == 0x00:
             _ignore, discharge_level, discharge_next_level, battery_status, voltage = _hidpp20.decipher_battery_status(n.data)
             status.set_battery_info(discharge_level, discharge_next_level, battery_status, voltage)
         elif n.address == 0x10:
-            if _log.isEnabledFor(_INFO):
-                _log.info('%s: spurious BATTERY status %s', device, n)
+            if logger.isEnabledFor(logging.INFO):
+                logger.info('%s: spurious BATTERY status %s', device, n)
         else:
-            _log.warn('%s: unknown BATTERY %s', device, n)
+            logger.warning('%s: unknown BATTERY %s', device, n)
 
     elif feature == _F.BATTERY_VOLTAGE:
         if n.address == 0x00:
-            _ignore, level, next, battery_status, voltage = _hidpp20.decipher_battery_voltage(n.data)
-            status.set_battery_info(level, next, battery_status, voltage)
+            _ignore, level, nextl, battery_status, voltage = _hidpp20.decipher_battery_voltage(n.data)
+            status.set_battery_info(level, nextl, battery_status, voltage)
         else:
-            _log.warn('%s: unknown VOLTAGE %s', device, n)
+            logger.warning('%s: unknown VOLTAGE %s', device, n)
 
     elif feature == _F.UNIFIED_BATTERY:
         if n.address == 0x00:
-            _ignore, level, next, battery_status, voltage = _hidpp20.decipher_battery_unified(n.data)
-            status.set_battery_info(level, next, battery_status, voltage)
+            _ignore, level, nextl, battery_status, voltage = _hidpp20.decipher_battery_unified(n.data)
+            status.set_battery_info(level, nextl, battery_status, voltage)
         else:
-            _log.warn('%s: unknown UNIFIED BATTERY %s', device, n)
+            logger.warning('%s: unknown UNIFIED BATTERY %s', device, n)
 
     elif feature == _F.ADC_MEASUREMENT:
         if n.address == 0x00:
             result = _hidpp20.decipher_adc_measurement(n.data)
             if result:
-                _ignore, level, next, battery_status, voltage = result
-                status.set_battery_info(level, next, battery_status, voltage)
+                _ignore, level, nextl, battery_status, voltage = result
+                status.set_battery_info(level, nextl, battery_status, voltage)
+            else:  # this feature is used to signal device becoming inactive
+                status.changed(active=False)
         else:
-            _log.warn('%s: unknown ADC MEASUREMENT %s', device, n)
+            logger.warning('%s: unknown ADC MEASUREMENT %s', device, n)
 
     elif feature == _F.SOLAR_DASHBOARD:
         if n.data[5:9] == b'GOOD':
@@ -361,8 +361,8 @@ def _process_feature_notification(device, status, n, feature):
                     status_text = _hidpp20.BATTERY_STATUS.recharging
                 status.set_battery_info(charge, None, status_text, None)
             elif n.address == 0x20:
-                if _log.isEnabledFor(_DEBUG):
-                    _log.debug('%s: Light Check button pressed', device)
+                if logger.isEnabledFor(logging.DEBUG):
+                    logger.debug('%s: Light Check button pressed', device)
                 status.changed(alert=_ALERT.SHOW_WINDOW)
                 # first cancel any reporting
                 # device.feature_request(_F.SOLAR_DASHBOARD)
@@ -371,72 +371,109 @@ def _process_feature_notification(device, status, n, feature):
                 reports_period = 2  # seconds
                 device.feature_request(_F.SOLAR_DASHBOARD, 0x00, reports_count, reports_period)
             else:
-                _log.warn('%s: unknown SOLAR CHARGE %s', device, n)
+                logger.warning('%s: unknown SOLAR CHARGE %s', device, n)
         else:
-            _log.warn('%s: SOLAR CHARGE not GOOD? %s', device, n)
+            logger.warning('%s: SOLAR CHARGE not GOOD? %s', device, n)
 
     elif feature == _F.WIRELESS_DEVICE_STATUS:
         if n.address == 0x00:
-            if _log.isEnabledFor(_DEBUG):
-                _log.debug('wireless status: %s', n)
+            if logger.isEnabledFor(logging.DEBUG):
+                logger.debug('wireless status: %s', n)
             reason = 'powered on' if n.data[2] == 1 else None
             if n.data[1] == 1:  # device is asking for software reconfiguration so need to change status
                 alert = _ALERT.NONE
                 status.changed(active=True, alert=alert, reason=reason, push=True)
         else:
-            _log.warn('%s: unknown WIRELESS %s', device, n)
+            logger.warning('%s: unknown WIRELESS %s', device, n)
 
     elif feature == _F.TOUCHMOUSE_RAW_POINTS:
         if n.address == 0x00:
-            if _log.isEnabledFor(_INFO):
-                _log.info('%s: TOUCH MOUSE points %s', device, n)
+            if logger.isEnabledFor(logging.INFO):
+                logger.info('%s: TOUCH MOUSE points %s', device, n)
         elif n.address == 0x10:
             touch = ord(n.data[:1])
             button_down = bool(touch & 0x02)
             mouse_lifted = bool(touch & 0x01)
-            if _log.isEnabledFor(_INFO):
-                _log.info('%s: TOUCH MOUSE status: button_down=%s mouse_lifted=%s', device, button_down, mouse_lifted)
+            if logger.isEnabledFor(logging.INFO):
+                logger.info('%s: TOUCH MOUSE status: button_down=%s mouse_lifted=%s', device, button_down, mouse_lifted)
         else:
-            _log.warn('%s: unknown TOUCH MOUSE %s', device, n)
+            logger.warning('%s: unknown TOUCH MOUSE %s', device, n)
 
     # TODO: what are REPROG_CONTROLS_V{2,3}?
     elif feature == _F.REPROG_CONTROLS:
         if n.address == 0x00:
-            if _log.isEnabledFor(_INFO):
-                _log.info('%s: reprogrammable key: %s', device, n)
+            if logger.isEnabledFor(logging.INFO):
+                logger.info('%s: reprogrammable key: %s', device, n)
         else:
-            _log.warn('%s: unknown REPROG_CONTROLS %s', device, n)
+            logger.warning('%s: unknown REPROG_CONTROLS %s', device, n)
+
+    elif feature == _F.BACKLIGHT2:
+        if (n.address == 0x00):
+            level = _unpack('!B', n.data[1:2])[0]
+            from solaar.ui.config_panel import record_setting  # prevent circular import
+            record_setting(device, _st.Backlight2Level, [level])
 
     elif feature == _F.REPROG_CONTROLS_V4:
         if n.address == 0x00:
-            if _log.isEnabledFor(_DEBUG):
+            if logger.isEnabledFor(logging.DEBUG):
                 cid1, cid2, cid3, cid4 = _unpack('!HHHH', n.data[:8])
-                _log.debug('%s: diverted controls pressed: 0x%x, 0x%x, 0x%x, 0x%x', device, cid1, cid2, cid3, cid4)
+                logger.debug('%s: diverted controls pressed: 0x%x, 0x%x, 0x%x, 0x%x', device, cid1, cid2, cid3, cid4)
         elif n.address == 0x10:
-            if _log.isEnabledFor(_DEBUG):
+            if logger.isEnabledFor(logging.DEBUG):
                 dx, dy = _unpack('!hh', n.data[:4])
-                _log.debug('%s: rawXY dx=%i dy=%i', device, dx, dy)
+                logger.debug('%s: rawXY dx=%i dy=%i', device, dx, dy)
         elif n.address == 0x20:
-            if _log.isEnabledFor(_DEBUG):
-                _log.debug('%s: received analyticsKeyEvents', device)
-        elif _log.isEnabledFor(_INFO):
-            _log.info('%s: unknown REPROG_CONTROLS_V4 %s', device, n)
+            if logger.isEnabledFor(logging.DEBUG):
+                logger.debug('%s: received analyticsKeyEvents', device)
+        elif logger.isEnabledFor(logging.INFO):
+            logger.info('%s: unknown REPROG_CONTROLS_V4 %s', device, n)
 
     elif feature == _F.HIRES_WHEEL:
         if (n.address == 0x00):
-            if _log.isEnabledFor(_INFO):
+            if logger.isEnabledFor(logging.INFO):
                 flags, delta_v = _unpack('>bh', n.data[:3])
                 high_res = (flags & 0x10) != 0
                 periods = flags & 0x0f
-                _log.info('%s: WHEEL: res: %d periods: %d delta V:%-3d', device, high_res, periods, delta_v)
+                logger.info('%s: WHEEL: res: %d periods: %d delta V:%-3d', device, high_res, periods, delta_v)
         elif (n.address == 0x10):
-            if _log.isEnabledFor(_INFO):
-                flags = ord(n.data[:1])
-                ratchet = flags & 0x01
-                _log.info('%s: WHEEL: ratchet: %d', device, ratchet)
+            ratchet = n.data[0]
+            if logger.isEnabledFor(logging.INFO):
+                logger.info('%s: WHEEL: ratchet: %d', device, ratchet)
+            if ratchet < 2:  # don't process messages with unusual ratchet values
+                from solaar.ui.config_panel import record_setting  # prevent circular import
+                record_setting(device, _st.ScrollRatchet, [2 if ratchet else 1])
         else:
-            if _log.isEnabledFor(_INFO):
-                _log.info('%s: unknown WHEEL %s', device, n)
+            if logger.isEnabledFor(logging.INFO):
+                logger.info('%s: unknown WHEEL %s', device, n)
+
+    elif feature == _F.ONBOARD_PROFILES:
+        if (n.address > 0x10):
+            if logger.isEnabledFor(logging.INFO):
+                logger.info('%s: unknown ONBOARD PROFILES %s', device, n)
+        else:
+            if (n.address == 0x00):
+                profile_sector = _unpack('!H', n.data[:2])[0]
+                if profile_sector:
+                    profile_change(device, profile_sector)
+            elif (n.address == 0x10):
+                resolution_index = _unpack('!B', n.data[:1])[0]
+                profile_sector = _unpack('!H', device.feature_request(_F.ONBOARD_PROFILES, 0x40)[:2])[0]
+                for profile in device.profiles.profiles.values() if device.profiles else []:
+                    if profile.sector == profile_sector:
+                        from solaar.ui.config_panel import record_setting  # prevent circular import
+                        record_setting(device, _st.AdjustableDpi, [profile.resolutions[resolution_index]])
 
     _diversion.process_notification(device, status, n, feature)
     return True
+
+
+# change UI to show result of onboard profile change
+def profile_change(device, profile_sector):
+    from solaar.ui.config_panel import record_setting  # prevent circular import
+    record_setting(device, _st.OnboardProfiles, [profile_sector])
+    for profile in device.profiles.profiles.values() if device.profiles else []:
+        if profile.sector == profile_sector:
+            resolution_index = profile.resolution_default_index
+            record_setting(device, _st.AdjustableDpi, [profile.resolutions[resolution_index]])
+            record_setting(device, _st.ReportRate, [profile.report_rate])
+            break
