@@ -16,6 +16,7 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+from logitech_receiver import base as _base
 from logitech_receiver import hidpp10 as _hidpp10
 from logitech_receiver import hidpp20 as _hidpp20
 from logitech_receiver import receiver as _receiver
@@ -81,10 +82,17 @@ def _battery_line(dev):
 
 def _print_device(dev, num=None):
     assert dev is not None
-    # check if the device is online
-    dev.ping()
+    # try to ping the device to see if it actually exists and to wake it up
+    try:
+        dev.ping()
+    except _base.NoSuchDevice:
+        print('  %s: Device not found' % num or dev.number)
+        return
 
-    print('  %d: %s' % (num or dev.number, dev.name))
+    if num or dev.number < 8:
+        print('  %d: %s' % (num or dev.number, dev.name))
+    else:
+        print('%s' % dev.name)
     print('     Device path  :', dev.path)
     if dev.wpid:
         print('     WPID         : %s' % dev.wpid)
@@ -97,7 +105,7 @@ def _print_device(dev, num=None):
     else:
         print('     Protocol     : unknown (device is offline)')
     if dev.polling_rate:
-        print('     Polling rate :', dev.polling_rate, 'ms (%dHz)' % (1000 // dev.polling_rate))
+        print('     Report Rate :', dev.polling_rate)
     print('     Serial number:', dev.serial)
     if dev.modelId:
         print('     Model ID:     ', dev.modelId)
@@ -212,8 +220,8 @@ def _print_device(dev, num=None):
                 if ids:
                     unitId, modelId, tid_map = ids
                     print('            Unit ID: %s  Model ID: %s  Transport IDs: %s' % (unitId, modelId, tid_map))
-            elif feature == _hidpp20.FEATURE.REPORT_RATE:
-                print('            Polling Rate (ms): %d' % _hidpp20.get_polling_rate(dev))
+            elif feature == _hidpp20.FEATURE.REPORT_RATE or feature == _hidpp20.FEATURE.EXTENDED_ADJUSTABLE_REPORT_RATE:
+                print('            Report Rate: %s' % _hidpp20.get_polling_rate(dev))
             elif feature == _hidpp20.FEATURE.REMAINING_PAIRING:
                 print('            Remaining Pairings: %d' % _hidpp20.get_remaining_pairing(dev))
             elif feature == _hidpp20.FEATURE.ONBOARD_PROFILES:
@@ -235,6 +243,8 @@ def _print_device(dev, num=None):
                         v = setting.val_to_string(setting.read(False))
                     except _hidpp20.FeatureCallError as e:
                         v = 'HID++ error ' + str(e)
+                    except AssertionError as e:
+                        v = 'AssertionError ' + str(e)
                     print('            %s        : %s' % (setting.label, v))
 
     if dev.online and dev.keys:
@@ -285,7 +295,6 @@ def run(devices, args, find_receiver, find_device):
     device_name = args.device.lower()
 
     if device_name == 'all':
-        dev_num = 1
         for d in devices:
             if isinstance(d, _receiver.Receiver):
                 _print_receiver(d)
@@ -293,17 +302,14 @@ def run(devices, args, find_receiver, find_device):
                 if count:
                     for dev in d:
                         print('')
-                        _print_device(dev)
+                        _print_device(dev, dev.number)
                         count -= 1
                         if not count:
                             break
                 print('')
             else:
-                if dev_num == 1:
-                    print('USB and Bluetooth Devices')
                 print('')
-                _print_device(d, num=dev_num)
-                dev_num += 1
+                _print_device(d)
         return
 
     dev = find_receiver(devices, device_name)
