@@ -1,5 +1,4 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2012-2013  Daniel Pavel
 ## Revisions Copyright (C) Contributors to the Solaar project.
@@ -18,8 +17,6 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 from gi.repository import Gtk
 from solaar import NAME, __version__
 from solaar.i18n import _
@@ -39,7 +36,7 @@ def _create():
     about.set_comments(_('Manages Logitech receivers,\nkeyboards, mice, and tablets.'))
     about.set_logo_icon_name(NAME.lower())
 
-    about.set_copyright('© 2012-2021 Daniel Pavel and contributors to the Solaar project')
+    about.set_copyright('© 2012-2023 Daniel Pavel and contributors to the Solaar project')
     about.set_license_type(Gtk.License.GPL_2_0)
 
     about.set_authors(('Daniel Pavel http://github.com/pwr', ))
@@ -78,6 +75,7 @@ def _create():
             'Daniel Pavel (română)',
             'Daniel Zippert, Emelie Snecker (svensk)',
             'Dimitriy Ryazantcev (Russian)',
+            'El Jinete Sin Cabeza (Español)',
         ))
     )
 
